@@ -148,6 +148,23 @@ Possible values:
 Related options:
     * None
 
+"""),
+    cfg.IntOpt('filesystem_store_chunk_size',
+               default=64 * units.Ki,
+               min=1,
+               help="""
+Chunk size, in bytes.
+
+The chunk size used when reading or writing image files. Raising this value
+may improve the throughput but it may also slightly increase the memory usage
+when handling a large number of requests.
+
+Possible Values:
+    * Any positive integer value
+
+Related options:
+    * None
+
 """)]
 
 MULTI_FILESYSTEM_METADATA_SCHEMA = {
@@ -243,8 +260,6 @@ class Store(glance_store.driver.Store):
                      capabilities.BitMasks.WRITE_ACCESS |
                      capabilities.BitMasks.DRIVER_REUSABLE)
     OPTIONS = _FILESYSTEM_CONFIGS
-    READ_CHUNKSIZE = 64 * units.Ki
-    WRITE_CHUNKSIZE = READ_CHUNKSIZE
     FILESYSTEM_STORE_METADATA = None
 
     def get_schemes(self):
@@ -384,19 +399,18 @@ class Store(glance_store.driver.Store):
         itself, it should raise `exceptions.BadStoreConfiguration`
         """
         if self.backend_group:
-            fdir = getattr(
-                self.conf, self.backend_group).filesystem_store_datadir
-            fdirs = getattr(
-                self.conf, self.backend_group).filesystem_store_datadirs
-            fstore_perm = getattr(
-                self.conf, self.backend_group).filesystem_store_file_perm
-            meta_file = getattr(
-                self.conf, self.backend_group).filesystem_store_metadata_file
+            store_conf = getattr(self.conf, self.backend_group)
         else:
-            fdir = self.conf.glance_store.filesystem_store_datadir
-            fdirs = self.conf.glance_store.filesystem_store_datadirs
-            fstore_perm = self.conf.glance_store.filesystem_store_file_perm
-            meta_file = self.conf.glance_store.filesystem_store_metadata_file
+            store_conf = self.conf.glance_store
+
+        fdir = store_conf.filesystem_store_datadir
+        fdirs = store_conf.filesystem_store_datadirs
+        fstore_perm = store_conf.filesystem_store_file_perm
+        meta_file = store_conf.filesystem_store_metadata_file
+
+        self.chunk_size = store_conf.filesystem_store_chunk_size
+        self.READ_CHUNKSIZE = self.chunk_size
+        self.WRITE_CHUNKSIZE = self.READ_CHUNKSIZE
 
         if not (fdir or fdirs):
             reason = (_("Specify at least 'filesystem_store_datadir' or "
@@ -436,7 +450,7 @@ class Store(glance_store.driver.Store):
                 (datadir_path,
                  priority) = self._get_datadir_path_and_priority(datadir)
                 priority_paths = self.priority_data_map.setdefault(
-                    int(priority), [])
+                    priority, [])
                 self._check_directory_paths(datadir_path, directory_paths,
                                             priority_paths)
                 directory_paths.add(datadir_path)
@@ -490,8 +504,9 @@ class Store(glance_store.driver.Store):
         parts = [part.strip() for part in datadir.rsplit(":", 1)]
         datadir_path = parts[0]
         if len(parts) == 2 and parts[1]:
-            priority = parts[1]
-            if not priority.isdigit():
+            try:
+                priority = int(parts[1])
+            except ValueError:
                 msg = (_("Invalid priority value %(priority)s in "
                          "filesystem configuration") % {'priority': priority})
                 LOG.exception(msg)
