@@ -24,6 +24,7 @@ import logging
 import math
 
 from oslo_config import cfg
+from oslo_utils import encodeutils
 from oslo_utils import units
 from six.moves import urllib
 
@@ -52,7 +53,7 @@ LOG = logging.getLogger(__name__)
 _RBD_OPTS = [
     cfg.IntOpt('rbd_store_chunk_size', default=DEFAULT_CHUNKSIZE,
                min=1,
-               help=_("""
+               help="""
 Size, in megabytes, to chunk RADOS images into.
 
 Provide an integer value representing the size in megabytes to chunk
@@ -70,9 +71,9 @@ Possible Values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.StrOpt('rbd_store_pool', default=DEFAULT_POOL,
-               help=_("""
+               help="""
 RADOS pool in which images are stored.
 
 When RBD is used as the storage backend for storing Glance images, the
@@ -90,9 +91,9 @@ Possible Values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.StrOpt('rbd_store_user', default=DEFAULT_USER,
-               help=_("""
+               help="""
 RADOS user to authenticate as.
 
 This configuration option takes in the RADOS user to authenticate as.
@@ -108,9 +109,9 @@ Possible Values:
 Related options:
     * rbd_store_ceph_conf
 
-""")),
+"""),
     cfg.StrOpt('rbd_store_ceph_conf', default=DEFAULT_CONFFILE,
-               help=_("""
+               help="""
 Ceph configuration file path.
 
 This configuration option takes in the path to the Ceph configuration
@@ -126,9 +127,9 @@ Possible Values:
 Related options:
     * rbd_store_user
 
-""")),
+"""),
     cfg.IntOpt('rados_connect_timeout', default=0,
-               help=_("""
+               help="""
 Timeout value for connecting to Ceph cluster.
 
 This configuration option takes in the timeout value in seconds used
@@ -144,7 +145,7 @@ Possible Values:
 Related options:
     * None
 
-"""))
+"""),
 ]
 
 
@@ -182,12 +183,7 @@ class StoreLocation(location.StoreLocation):
 
     def parse_uri(self, uri):
         prefix = 'rbd://'
-        if not uri.startswith(prefix):
-            reason = _('URI must start with rbd://')
-            msg = _LI("Invalid URI: %s") % reason
-
-            LOG.info(msg)
-            raise exceptions.BadStoreUri(message=reason)
+        self.validate_schemas(uri, valid_schemas=(prefix,))
         # convert to ascii since librbd doesn't handle unicode
         try:
             ascii_uri = str(uri)
@@ -246,7 +242,7 @@ class ImageIterator(object):
                             data = image.read(size - bytes_left, length)
                             bytes_left -= len(data)
                             yield data
-                        raise StopIteration()
+                        return
         except rbd.ImageNotFound:
             raise exceptions.NotFound(
                 _('RBD image %s does not exist') % self.name)
@@ -286,17 +282,32 @@ class Store(driver.Store):
         itself, it should raise `exceptions.BadStoreConfiguration`
         """
         try:
-            chunk = self.conf.glance_store.rbd_store_chunk_size
+            if self.backend_group:
+                chunk = getattr(self.conf,
+                                self.backend_group).rbd_store_chunk_size
+                pool = getattr(self.conf, self.backend_group).rbd_store_pool
+                user = getattr(self.conf, self.backend_group).rbd_store_user
+                conf_file = getattr(self.conf,
+                                    self.backend_group).rbd_store_ceph_conf
+                connect_timeout = getattr(
+                    self.conf, self.backend_group).rados_connect_timeout
+            else:
+                chunk = self.conf.glance_store.rbd_store_chunk_size
+                pool = self.conf.glance_store.rbd_store_pool
+                user = self.conf.glance_store.rbd_store_user
+                conf_file = self.conf.glance_store.rbd_store_ceph_conf
+                connect_timeout = self.conf.glance_store.rados_connect_timeout
+
             self.chunk_size = chunk * units.Mi
             self.READ_CHUNKSIZE = self.chunk_size
             self.WRITE_CHUNKSIZE = self.READ_CHUNKSIZE
 
             # these must not be unicode since they will be passed to a
             # non-unicode-aware C library
-            self.pool = str(self.conf.glance_store.rbd_store_pool)
-            self.user = str(self.conf.glance_store.rbd_store_user)
-            self.conf_file = str(self.conf.glance_store.rbd_store_ceph_conf)
-            self.connect_timeout = self.conf.glance_store.rados_connect_timeout
+            self.pool = str(pool)
+            self.user = str(user)
+            self.conf_file = str(conf_file)
+            self.connect_timeout = connect_timeout
         except cfg.ConfigFileValueError as e:
             reason = _("Error in store configuration: %s") % e
             LOG.error(reason)
@@ -353,7 +364,7 @@ class Store(driver.Store):
 
         :param image_name: Image's name
 
-        :retval: `glance_store.rbd.StoreLocation` object
+        :returns: `glance_store.rbd.StoreLocation` object
         """
         librbd = rbd.RBD()
         features = conn.conf_get('rbd_default_features')
@@ -430,8 +441,9 @@ class Store(driver.Store):
             # Such exception is not dangerous for us so it will be just logged
             LOG.debug("Snapshot %s is unprotected already" % snap_name)
 
+    @driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -441,20 +453,35 @@ class Store(driver.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
+        :param context: A context object
         :param verifier: An object used to verify signatures for images
 
-        :retval: tuple of URL in backing store, bytes written, checksum
-                and a dictionary with storage system specific information
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
         :raises: `glance_store.exceptions.Duplicate` if the image already
-                existed
+                 exists
         """
         checksum = hashlib.md5()
+        os_hash_value = hashlib.new(str(hashing_algo))
         image_name = str(image_id)
         with self.get_connection(conffile=self.conf_file,
                                  rados_id=self.user) as conn:
             fsid = None
             if hasattr(conn, 'get_fsid'):
-                fsid = conn.get_fsid()
+                # Librados's get_fsid is represented as binary
+                # in py3 instead of str as it is in py2.
+                # This is causing problems with ceph.
+                # Decode binary to str fixes these issues.
+                # Fix with encodeutils.safe_decode CAN BE REMOVED
+                # after librados's fix will be stable.
+                #
+                # More informations:
+                # https://bugs.launchpad.net/glance-store/+bug/1816721
+                # https://bugs.launchpad.net/cinder/+bug/1816468
+                # https://tracker.ceph.com/issues/38381
+                fsid = encodeutils.safe_decode(conn.get_fsid())
             with conn.open_ioctx(self.pool) as ioctx:
                 order = int(math.log(self.WRITE_CHUNKSIZE, 2))
                 LOG.debug('creating image %s with order %d and size %d',
@@ -492,12 +519,28 @@ class Store(driver.Store):
                             LOG.debug(_("writing chunk at offset %s") %
                                       (offset))
                             offset += image.write(chunk, offset)
+                            os_hash_value.update(chunk)
                             checksum.update(chunk)
                             if verifier:
                                 verifier.update(chunk)
                         if loc.snapshot:
                             image.create_snap(loc.snapshot)
                             image.protect_snap(loc.snapshot)
+                except rbd.NoSpace:
+                    log_msg = (_LE("Failed to store image %(img_name)s "
+                                   "insufficient space available") %
+                               {'img_name': image_name})
+                    LOG.error(log_msg)
+
+                    # Delete image if one was created
+                    try:
+                        target_pool = loc.pool or self.pool
+                        self._delete_image(target_pool, loc.image,
+                                           loc.snapshot)
+                    except exceptions.NotFound:
+                        pass
+
+                    raise exceptions.StorageFull(message=log_msg)
                 except Exception as exc:
                     log_msg = (_LE("Failed to store image %(img_name)s "
                                    "Store Exception %(store_exc)s") %
@@ -519,7 +562,16 @@ class Store(driver.Store):
         if image_size == 0:
             image_size = bytes_written
 
-        return (loc.get_uri(), image_size, checksum.hexdigest(), {})
+        # Add store backend information to location metadata
+        metadata = {}
+        if self.backend_group:
+            metadata['backend'] = u"%s" % self.backend_group
+
+        return (loc.get_uri(),
+                image_size,
+                checksum.hexdigest(),
+                os_hash_value.hexdigest(),
+                metadata)
 
     @capabilities.check
     def delete(self, location, context=None):
